#!/usr/bin/wish

# Copyright (c) 2006-2014, Botik Technologies, Ltd.
# See license.txt for more information.

#   BotikLook,  BotikLook v.3 (http://www.botik.ru/~botik/look/).
#   : 1  2006 .

proc initManager {} {
    global tcl_platform blook etab shed env
    if {$tcl_platform(os) ne "Windows NT"} {
        alertError "      Windows NT (2000/XP/2003/Vista).\n   ."
        exit
    }
    set env(APPDATA)       .
    set blook(progname)       "BotikLook"
    set blook(prjname)       "$blook(progname)"
    set blook(modname)     "$blook(prjname) Service Manager"
    set blook(modver)      "1.01"
    set blook(pname)       "$blook(modname) v.$blook(modver)"
    set blook(svcname)     "blsvc"
    set blook(svcdispname) "$blook(progname) Service"
    set blook(icon)           [file join img botiklook.ico]
    set blook(svcscript)   [file normalize $blook(prjname).tcl]
    set blook(svcrun)      0; # -   
    set blook(inipath)     [appFile init.cfg]
    set blook(sc)           sc.exe
    set blook(svcKey)      HKEY_LOCAL_MACHINE\\SYSTEM\\CurrentControlSet\\Services\\$blook(svcname)
    set blook(svcEvntKey)  HKEY_LOCAL_MACHINE\\SYSTEM\\CurrentControlSet\\Services\\Eventlog\\Application\\$blook(svcname)
    set blook(paused)       0
    set blook(uto)           5000
    set blook(mgrport)       5000
    set blook(svcport)     5001
    set blook(key)         [binary format H* 0123456789abcdefabcdef01234567890123456789abcdefabcdef0123456789]
    set etab(n)               0
    set etab(id)           0
    set etab(rb)           {}
    set shed(allstrtypes)  {"" "" ""}
    if {[file exists [file join .. bin $blook(svcname).exe]]} {
        set blook(svcexe) [file join .. bin $blook(svcname).exe]
    } elseif {[file exists [file join bin $blook(svcname).exe]]} {
        set blook(svcexe) [file join bin $blook(svcname).exe]
    } else {
        alertError "    \"$blook(svcname).exe\".\n   ."
        exit
    }
    wm title . $blook(pname)
    wm state . withdrawn
    setIcon
    setDefaults
    readIniFile
    startServer
    startEventLoop
}
#    .   ,
#    . .
proc appFile {{fname {}}} {
    set d [expr {([info exists ::env(APPDATA)]) ? $::env(APPDATA) : {}}]
    file mkdir [file join $d $::blook(prjname)]
    set appfile [file join $d $::blook(prjname) $fname]
    if {![lempty $fname] && ![file exists $appfile]} {touch $appfile}
    return $appfile
}
#     
proc touch {file} {
    close [open $file w]
}
proc loadPackages {plist} {
    # packages - . ,    -   ,   -  .
    set notsat {}
    foreach {pac ver} $plist {
        #    ,      
        if {![catch {package present $pac}]} {continue}
        if {[catch {package require $pac} eid]} {
            alertError ":     \"$pac\".\n$eid\n\   ."
            exit
        } elseif {$pac eq "tls"} {
            http::register https 443 ::tls::socket
        } elseif {$pac eq "style"} {
            style::use as
        }
        if {![regexp {\S} $ver]} {continue}
        #   ,    .
        #    ,     .
        set versions [package versions $pac]
        set sat 0
        foreach v $versions {
            if {[package vsatisfies $v $ver]} {set sat 1}
        }
        if {!$sat} {lappend notsat $pac $ver}
    }
    if {![lempty $notsat]} {
        if {[llength $notsat]==2} {
            alertWarning "!   [lindex $notsat 0]\n   ([lindex $notsat 1]).\n   ."
            return
        }
        set packs {}
        set verss {}
        foreach {pac ver} $notsat {
            lappend packs $pac
            lappend verss $ver
        }
        alertWarning "!   $packs\n   ($verss).\n   ."
        return
    }
}
proc alert {msg} {
    tk_messageBox -icon info -type ok -title "" -message "$msg"
}
proc alertError {msg} {
    tk_messageBox -icon error -type ok -title "" -message "$msg"
}
proc alertWarning {msg} {
    tk_messageBox -icon warning -type ok -title "" -message $msg
}
proc alertQuestion {msg} {
    return [tk_messageBox -icon question -type yesno -default yes -title "" -message $msg]
}
proc reallyExit {} {
    return 1
    #set q "     ?"
    #return [expr {[alertQuestion $q] eq "yes"}]
}
proc doExit {{ask 1}} {
    if {$ask && ![reallyExit]} {return}
    catch {winico delete [winico info]}
    exit
}
proc initMenu {} {
    catch {destroy .m}
    menu .m -tearoff 0
    set state [serviceStatus]
    .m add command -label "" -command "startService"
    .m add command -label ""  -command "stopService"
    .m add command -label "" -command "pauseService"
    .m add command -label "" -command "restartService"
    .m add command -label "" -command "syncNow"
    switch -- $state {
        "running" {
            .m entryconfigure 0 -state disabled
            if {[isPaused]} {
                .m entryconfigure 1 -state disabled
                .m entryconfigure 2 -label "" -command "resumeService"
                .m entryconfigure 3 -state disabled
                .m entryconfigure 4 -state disabled
            }
        }
        "start_pending" - "stop_pending" {
            .m entryconfigure 0 -state disabled
            .m entryconfigure 1 -state disabled
            .m entryconfigure 2 -state disabled
            .m entryconfigure 3 -state disabled
            .m entryconfigure 4 -state disabled
        }
        "notexists" - "stopped" {
            .m entryconfigure 1 -state disabled
            .m entryconfigure 2 -state disabled
            .m entryconfigure 3 -state disabled
            .m entryconfigure 4 -state disabled
        }
        default {}
    }
    .m add command -label "" -command "showConfig"
    .m add command -label ""   -command "showHelp"
    .m add command -label ""     -command "doExit"
    return .m
}
proc tray_cmd {winmsg x y} {
    if {$winmsg eq "WM_LBUTTONUP"} {
        showConfig
    } elseif {$winmsg eq "WM_RBUTTONUP"} {
        [initMenu] post $x $y
    }
}
proc findTaskbarIconsPos {} {
    global botikkey positions
    set ico         [winico info]
    set info        [winico info $ico]
    set screendepth [winfo screendepth .]
    set smallsize   "16x16"
    set bigsize     "32x32"
    set smallpos    -1
    set bigpos      -1
    set smalldepth  0
    set bigdepth    0
    foreach i $info {
        array set opts $i
        set depth    $opts(-bpp)
        set pos      $opts(-pos)
        set geometry $opts(-geometry)
        if {$geometry eq "$bigsize" && $depth <= $screendepth} {
            if {$depth > $bigdepth} {
                set bigpos   $pos
                set bigdepth $depth
            }
        } elseif {$geometry eq "$smallsize" && $depth <= $screendepth} {
            if {$depth > $smalldepth} {
                set smallpos   $pos
                set smalldepth $depth
            }
        }
    }
    return [list $smallpos $bigpos]
}
#      
proc setIcon {} {
    winico createfrom $::blook(icon)
    trayIcon add
}
proc trayIcon {cmd} {
    set pos  0
    set text $::blook(pname)
    catch {set pos [lindex [findTraybarIconPos] 0]}
    if {$cmd eq "add"} {
        winico taskbar add [winico info] -callback "tray_cmd %m %x %y" -text $text -pos $pos
    } else {
        winico taskbar modify [winico info] -text $text -pos $pos
    }
}
proc modifyTrayText {text} {
    winico taskbar modify [winico info] -text $text
}
proc modifyTrayIcon {newidx} {
    winico taskbar modify [winico info] -pos $newidx
}
proc doSource {file {globals {}}} {
    if {![file exists $file]} {return}
    foreach var $globals {
        global $var
    }
    for_file line $file {
        if {[lindex $line 0] ne "set"} {continue}
        eval [subst -nocommands -novariables $line]
    }
}
proc installService {} {
    if {[catch {
        exec $::blook(svcexe) -install $::blook(svcdispname)
        registry set $::blook(svcKey)     "TclScript"        "source {$::blook(svcscript)}"
        registry set $::blook(svcEvntKey) "EventMessageFile" [file nativename $::blook(svcexe)]
        registry set $::blook(svcEvntKey) "TypesSupported"   7 "dword"
    } desc]} {
        alertError ":    .\n\n$desc"
        return 0
    }
    return 1
}
proc uninstallService {} {
    exec $::blook(svcexe) -remove
    registry delete $::blook(svcKey)
    registry delete $::blook(svcEvntKey)
}
#    : auto_start,demand_start or disabled.
proc getStartType {} {
    if {[catch {set res [exec $::blook(sc) qc $::blook(svcname)]} desc]} {
        alertError " .\n\n$desc"
        return
    }
    set res [string map {\n "|"} $res]
    set res [split $res {|}]
    set info [lindex $res 0]
    if {[string equal -nocase "1060:" [lindex $info end]]} {
        return
    }
    foreach line $res {
        foreach {par val} [split $line {:}] {}
        if {[string equal -nocase [trim $par] "start_type"]} {
            return [string tolower [trim [lindex $val end]]]
        }
    }
}
#   : notexists, stopped, running, stop_pending, start_pending.
#   sc.exe.
proc serviceStatus {} {
    if {[catch {set res [exec $::blook(sc) query $::blook(svcname)]} desc]} {
        alertError " .\n\n$desc"
        return "notexists"
    }
    set res [string map {\n "|"} $res]
    set res [split $res {|}]
    set info [lindex $res 0]
    if {[string equal -nocase "1060:" [lindex $info end]]} {
        return "notexists"
    }
    foreach line $res {
        foreach {par val} [split $line {:}] {
            if {[string equal -nocase [trim $par] "state"]} {
                return [string tolower [trim [lindex $val end]]]
            }
        }
    }
    return "notexists"
}
proc trim {str} {
    return [string trim $str]
}
proc readIniFile {} {
    global stats botiklook
    #       $blook(inipath).
    catch {doSource $::blook(inipath) [list stats botiklook]}
    set state [serviceStatus]
    #   
    if {[info exists ::blook(svccfgpath)]} {
        #   
        if {$state ne "notexists"} {
            #     ,   
            if {$state eq "stopped"} {
                #set msg " $::blook(progname) .\n    ?"
                #if {[alertQuestion $msg] eq "yes"} {
                #    startService
                #}
            }
            return
        }
    } else {
        #   
        if {$state ne "notexists"} {
            #         
            alertError ":   $::blook(progname)\n     .\n      ."
            doExit 0
        }
    }
    showConfig
}
proc restartService {} {
    if {[isPaused]} {resumeService}
    stopService
    after 1000 startService; # it doesn't stop instantly, have to wait
}
proc startService {} {
    set state [serviceStatus]
    if {$state eq "running"} {
        #alert "  ."
        return
    }
    if {$state eq "notexists"} {
        alert "   ."
        showConfig
        return
    }
    # 1.     .
    if {[getStartType] eq "disabled"} {
        if {$::shed(auto) && [catch {setStartType "auto"} desc]} {
            alertError " .\n\n$desc"
            return
        } elseif {!$::shed(auto) && [catch {setStartType "demand"} desc]} {
            alertError " .\n\n$desc"
            return
        }
    }
    # 2. .
    if {$state eq "stopped"} {
        if {[catch {set res [exec $::blook(sc) start $::blook(svcname)]} desc]} {
            alertError " .\n\n$desc"
            return
        }
        #alert "  ."
    }
    return
}
proc setStartType {type} {
    set res [exec $::blook(sc) config $::blook(svcname) start= $type]
}
#  
proc createService {} {
    set state [serviceStatus]
    if {$state eq "notexists"} {
        installService
        startService
    } else {
        set st [getStartType]
        if {$st eq "auto_start" && !$::shed(auto)} {
            setStartType "demand"
        } elseif {$st eq "demand_start" && $::shed(auto)} {
            setStartType "auto"
        }
    }
}
proc stopService {} {
    set state [serviceStatus]
    if {$state eq "running"} {
        if {[catch {set res [exec $::blook(sc) stop $::blook(svcname)]} desc]} {
            alertError " .\n\n$desc"
            return
        }
    } elseif {$state eq "stopped"} {
        alert "  ."
    }
    return
}
proc pauseService {} {
    sendMsg "pause"
}
proc resumeService {} {
    sendMsg "resume"
}
proc syncNow {} {
    sendMsg "syncnow"
}
proc isPaused {} {
    if {![catch {sendMsg "ispaused"} desc]} {
        vwait ::blook(paused)
    } else {
        return 0
    }
    return $::blook(paused)
}
#     
proc showHelp {} {
    global blook env
    set chmhelpfile  [file join . doc $blook(progname).chm]
    set chmpath      "html/service.html"
    try_eval {
        #package require winhelp 1.0
        exec hh.exe doc/$blook(progname).chm::$chmpath
        winico taskbar modify [winico info] -text "  "
        #winhelp . $chmhelpfile $chmpath
    } {
        alertError "    .\n :\n$errorInfo"
    }
}
proc getTimeStr {type} {
    return [lindex $::shed(allstrtypes) $type]
}
proc setShedType {} {
    if {$::shed(strtype) eq ""} {
        set ::shed(type) 0
    } elseif {$::shed(strtype) eq ""} {
        set ::shed(type) 1
    } elseif {$::shed(strtype) eq ""} {
        set ::shed(type) 2
    }
}
proc readSvcConfig {} {
    catch {doSource $::blook(svccfgpath)}
    setDefaults
}
proc checkOrg {org} {
    return [expr ![lempty $org] && [regexp {^[a-zA-Z0-9_]*$} $org]]
}
proc checkLogin {login} {
    return [expr ![lempty $login] && [regexp {^[_a-zA-Z0-9][-_a-zA-Z0-9]*$} $login]]
}
proc showConfig {} {
    global auth
    if {[file size $::blook(inipath)] == 0} {
        alert "      $::blook(progname)."
        return
    }
    wm iconbitmap . -default $::blook(icon)
    if {[winfo ismapped .]} {
        wm state . withdrawn
    } else {
        wm state . normal
        focus -force .
    }
    if {[winfo exists .left]} {return}
    #bind . <Unmap> "wm state . withdrawn"
    bind . <Escape> [bind . <Unmap>]
    wm protocol  . WM_DELETE_WINDOW {
        wm state . withdrawn
    }

    readSvcConfig

    frame .left
    frame .right
    pack .left .right -side left -fill y -expand 1

    frame .left.account
    frame .left.table
    frame .left.opts
    pack .left.account .left.table .left.opts -side top -fill both -expand 1

    set af .left.account
    set tab .left.table
    frame $af.f1
    frame $af.f2
    frame $af.fol
    frame $af.f3
    pack $af.f1 $af.f2 $af.fol $af.f3 -fill both
    label $af.f1.label -text ",     \n      \"$::net(name)\"."\
        -font {Helvetica 12 bold}
    pack $af.f1.label -expand 1 -padx 5 -pady 10
    frame $af.f2.labels
    frame $af.f2.entries
    pack $af.f2.labels $af.f2.entries -side left -expand 1
    set l $af.f2.labels
    set e $af.f2.entries
    label $l.org   -text "  :" -anchor w -font {Helvetica 11}
    label $l.login -text " :"     -anchor w -font {Helvetica 11}
    label $l.pswd  -text " :"       -anchor w -font {Helvetica 11}
    pack $l.org $l.login $l.pswd -pady 2 -padx 3 -fill x
    entry $e.org   -width 20 -takefocus 1 -textvar auth(org) -validate key -vcmd {checkOrg %P}
    entry $e.login -width 20 -takefocus 1 -textvar auth(login) -validate key -vcmd {checkLogin %P}
    entry $e.pswd  -width 20 -takefocus 1 -textvar auth(password) -show "*" ; #         ,        
    pack $e.org $e.login $e.pswd -pady 4
    set auth(password) ""
    if {![info exists auth(org)] || [lempty $auth(org)]} {
        focus -force $e.org
    } elseif {![info exists auth(login)] || [lempty $auth(login)]} {
        focus -force $e.login
    } elseif {![info exists auth(password)] || [lempty $auth(password)]} {
        focus -force $e.pswd
    }
    bind $e.org   <KeyPress-Return> "$tab.buts.badd invoke"
    bind $e.login <KeyPress-Return> "$tab.buts.badd invoke"
    bind $e.pswd  <KeyPress-Return> "$tab.buts.badd invoke"

    frame $tab.data
    frame $tab.buts
    pack $tab.data $tab.buts -side left -fill both -expand 1

    labelframe $tab.data.lf -text " " -font {Helvetica 11}
    pack $tab.data.lf -side top -fill both -expand 1

    set lf $tab.data.lf
    frame $lf.accounts
    frame $lf.radios
    pack $lf.accounts $lf.radios -side left -expand 1 -fill both
    
    button $tab.buts.badd -text "" -command entryAdd -font {Helvetica 11} -width 8 -default active
    button $tab.buts.bchange -text "" -command entryChange -font {Helvetica 11} -width 8
    button $tab.buts.bdel -text "" -command entryDel -font {Helvetica 11} -width 8
    button $tab.buts.bup -text "" -command entryUp -font {Helvetica 11} -width 8
    button $tab.buts.bdown -text "" -command entryDown -font {Helvetica 11} -width 8
    pack $tab.buts.badd $tab.buts.bchange $tab.buts.bdel $tab.buts.bup $tab.buts.bdown -side top -fill none

    addEntries

    set opts .left.opts

    checkbutton $opts.cb -text "" -font {Helvetica 11} -variable ::shed(sync) -anchor w -command changeSlavesState
    labelframe $opts.lf -labelwidget $opts.cb
    pack $opts.lf -side top -fill both -expand 1
    label $opts.lf.l -text ":"
    tk_optionMenu $opts.lf.menu ::shed(strtype) "" "" ""
    frame $opts.lf.dframe
    pack $opts.lf.l $opts.lf.menu -side left
    pack $opts.lf.dframe -side left -fill x -expand 1

    set dframe $opts.lf.dframe; # dynamic frame
    setPeriodWidget $dframe {} {} {}
    trace add variable ::shed(strtype) write "setPeriodWidget $dframe"

    labelframe $opts.lff -text "  " -font {Helvetica 10}
    pack $opts.lff -fill x

    entry $opts.lff.e -textvariable ::blook(svccfgpath) -width 40 -state readonly
    button $opts.lff.b -text "..." -font {Helvetica 10} -command "chooseFile"
    pack $opts.lff.e $opts.lff.b -side left -padx 3

    checkbutton $opts.cba -text "    " -variable ::shed(auto) -anchor w
    pack $opts.cba -fill x

    frame $opts.buts
    pack $opts.buts -fill x -expand 1
    button $opts.buts.bok -text "OK" -width 8 -font {Helvetica 10 bold} -command saveSettings
    button $opts.buts.bapp -text ""  -font {Helvetica 10 bold} -command {saveSettings 0}
    button $opts.buts.canc -text "" -font {Helvetica 10 bold} -command {wm state . withdrawn}
    pack $opts.buts.bok $opts.buts.bapp $opts.buts.canc -side left -expand 1 -pady 5
}
proc changeSlavesState {} {
    set opts .left.opts
    set dframe $opts.lf.dframe
    set slist "$opts.lf.l $opts.lf.menu"
    if {$::shed(type) != 2} {
        set slist "$slist [pack slaves $opts.lf.dframe]"
        foreach slave $slist {
            if {$::shed(sync)} {
                $slave configure -state normal
            } else {
                $slave configure -state disabled
            }
        }
        return
    }
    set slist "$slist [pack slaves $dframe.f1] [pack slaves $dframe.f2] [pack slaves $dframe.f3]"
    foreach slave $slist {
        if {$::shed(sync)} {
            $slave configure -state normal
        } else {
            $slave configure -state disabled
        }
    }
}
#     
proc entryAdd {{org {none}} {login {none}} {password {none}}} {
    if {$org eq "none" && $login eq "none" && $password eq "none"} {
        if {!([checkOrg $::auth(org)] && [checkLogin $::auth(login)])} {
            alertError "    ."
            return
        }
        set org $::auth(org)
        set login $::auth(login)
        set password          [encrypt $::auth(password)]
        set ::auth(org)      ""
        set ::auth(login)    ""
        set ::auth(password) ""
        focus -force .left.account.f2.entries.org
    }
    set ::etab(vals,$::etab(id)) [list $org $login $password]
    set mfr .left.table.data.lf
    label $mfr.accounts.l$::etab(id) -text "\[$org\] $login" -pady 2 -anchor w
    radiobutton $mfr.radios.r$::etab(id) -variable ::etab(rb) -value $::etab(id) -anchor e
    pack $mfr.accounts.l$::etab(id) $mfr.radios.r$::etab(id) -fill x
    set ::etab(rb) $::etab(id)
    incr ::etab(n)
    incr ::etab(id)
}
#     
proc entryDel {} {
    if {!$::etab(n)} {return}
    set mfr .left.table.data.lf
    #  
    pack forget $mfr.accounts.l$::etab(rb) $mfr.radios.r$::etab(rb)
    destroy $mfr.accounts.l$::etab(rb) $mfr.radios.r$::etab(rb)
    unset ::etab(vals,$::etab(rb))
    incr ::etab(n) -1
    set ::etab(rb) [nextEntry]
}
#    :
#     .  ,    
proc entryChange {} {
    if {!$::etab(n)} {return}
    set entry [lrange $::etab(vals,$::etab(rb)) 0 1]
    foreach {::auth(org) ::auth(login)} $entry {}
    entryDel
}
#   ()   
proc nextEntry {} {
    set names [array names ::etab vals,*]
    foreach name $names {
        set idx [lindex [split $name ","] end]
        lappend remlist $idx
    }
    if {[info exists remlist]} {
        set remlist [lsort -integer $remlist]
        foreach rem $remlist {
            if {$rem > $::etab(rb)} {
                return $rem
            }
        }
        return [lindex $remlist end]
    }
    return 0
}
#   ()   
proc prevEntry {} {
    set names [array names ::etab vals,*]
    foreach name $names {
        set idx [lindex [split $name ","] end]
        lappend remlist $idx
    }
    if {[info exists remlist]} {
        set remlist [lsort -integer -decreasing $remlist]
        foreach rem $remlist {
            if {$rem < $::etab(rb)} {
                return $rem
            }
        }
        return [lindex $remlist 0]
    }
    return 0
}
proc entryUp {} {
    set prevn   [prevEntry]
    if {$::etab(rb) <= $prevn} {return}
    set val     $::etab(vals,$::etab(rb))
    set prevval $::etab(vals,$prevn)
    set ::etab(vals,$prevn)      $val
    set ::etab(vals,$::etab(rb)) $prevval
    set mfr .left.table.data.lf
    $mfr.accounts.l$prevn configure -text "\[[lindex $val 0]\] [lindex $val 1]"
    $mfr.accounts.l$::etab(rb) configure -text "\[[lindex $prevval 0]\] [lindex $prevval 1]"
    set ::etab(rb) $prevn
}
proc entryDown {} {
    set nextn   [nextEntry]
    if {$::etab(rb) >= $nextn} {return}
    set val     $::etab(vals,$::etab(rb))
    set nextval $::etab(vals,$nextn)
    set ::etab(vals,$nextn)      $val
    set ::etab(vals,$::etab(rb)) $nextval
    set mfr .left.table.data.lf
    $mfr.accounts.l$nextn configure -text "\[[lindex $val 0]\] [lindex $val 1]"
    $mfr.accounts.l$::etab(rb) configure -text "\[[lindex $nextval 0]\] [lindex $nextval 1]"
    set ::etab(rb) $nextn
}
proc debug {str} {
    set f [open debug.txt a]
    puts $f $str
    close $f
}
proc setPeriodWidget {dframe n1 n2 {op 0}} {
    set oldtype $::shed(type)
    setShedType
    if {![lempty $op] && $::shed(type) == $oldtype} {return}
    pack forget $dframe
    destroy $dframe
    frame $dframe
    pack $dframe -side left -fill x -expand 1
    #        
    if {$::shed(type) == 0} {
        label $dframe.l1 -text ""
        entry $dframe.hour -textvariable ::shed(hour) -width 2 \
            -validate key -vcmd {checkHour %P} -relief flat
        label $dframe.l2 -text ""
        pack $dframe.l1 $dframe.hour $dframe.l2 -side left
        bind $dframe.hour <FocusOut> "setDefaults"
        bind $dframe.hour <FocusIn>  "%W selection range 0 end"
    } elseif {$::shed(type) == 1} {
        label $dframe.l1 -text ""
        entry $dframe.day -textvariable ::shed(day) -width 2 \
            -validate key -vcmd {checkDay %P} -relief flat
        label $dframe.l2 -text ""
        label $dframe.l3 -text ""
        entry $dframe.hour -textvariable ::shed(timeh) -width 2 \
            -validate key -vcmd {checkHour2 %P} -relief flat
        label $dframe.l4 -text "."
        entry $dframe.min -textvariable ::shed(timem) -width 2 \
            -validate key -vcmd {checkMinutes %P} -relief flat
        label $dframe.l5 -text "."
        pack $dframe.l1 $dframe.day $dframe.l2 $dframe.l3 $dframe.hour $dframe.l4 $dframe.min $dframe.l5 -side left
        bind $dframe.day  <FocusOut> "setDefaults"
        bind $dframe.hour <FocusOut> "setDefaults"
        bind $dframe.min  <FocusOut> "setDefaults"
        bind $dframe.day  <FocusIn>  "%W selection range 0 end"
        bind $dframe.hour <FocusIn>  "%W selection range 0 end"
        bind $dframe.min  <FocusIn>  "%W selection range 0 end"
    } elseif {$::shed(type) == 2} {
        frame $dframe.f1
        frame $dframe.f2
        frame $dframe.f3
        pack $dframe.f1 $dframe.f2 $dframe.f3
        label $dframe.f1.l1 -text ""
        entry $dframe.f1.weekn -textvariable ::shed(week) -width 1 \
            -validate key -vcmd {checkWeek %P} -relief flat
        label $dframe.f1.l2 -text ""
        label $dframe.f1.l3 -text ""
        pack $dframe.f1.l1 $dframe.f1.weekn $dframe.f1.l2 $dframe.f1.l3 -side left
        set weeks {. .  . . . .}
        loop i 0 3 {
            set week [lindex $weeks $i]
            checkbutton $dframe.f1.weekday$i -text $week -variable ::shed(weekdays,$i) -relief flat -overrelief raised
            pack $dframe.f1.weekday$i -side left
        }
        loop i 3 7 {
            set week [lindex $weeks $i]
            checkbutton $dframe.f2.weekday$i -text $week -variable ::shed(weekdays,$i) -relief flat -overrelief raised
            pack $dframe.f2.weekday$i -side left
        }
        label $dframe.f3.l3 -text ""
        entry $dframe.f3.hour -textvariable ::shed(timeh) -width 2 \
            -validate key -vcmd {checkHour2 %P} -relief flat
        label $dframe.f3.l4 -text "."
        entry $dframe.f3.min -textvariable ::shed(timem) -width 2 \
            -validate key -vcmd {checkMinutes %P} -relief flat
        label $dframe.f3.l5 -text "."
        pack $dframe.f3.l3 $dframe.f3.hour $dframe.f3.l4 $dframe.f3.min $dframe.f3.l5 -side left
        bind $dframe.f1.weekn <FocusOut> "setDefaults"
        bind $dframe.f3.hour  <FocusOut> "setDefaults"
        bind $dframe.f3.min   <FocusOut> "setDefaults"
        bind $dframe.f1.weekn <FocusIn>  "%W selection range 0 end"
        bind $dframe.f3.hour  <FocusIn>  "%W selection range 0 end"
        bind $dframe.f3.min   <FocusIn>  "%W selection range 0 end"
    }
}
#  ,     
proc setDefaults {} {
    # def -      shed
    foreach idx {type hour timeh timem day week sync auto} def {0 24 0 0 1 1 1 1} {
        if {![info exists ::shed($idx)] || [lempty $::shed($idx)]} {set ::shed($idx) $def}
    }
    set sum 0
    loop i 0 7 {
        if {$i == 4} {set val 1} else {set val 0}
        set ::shed(weekdays,$i) [expr {(![info exists ::shed(weekdays,$i)]) ? $val : $::shed(weekdays,$i)}]
        incr sum $::shed(weekdays,$i)
    }
    if {$sum == 0} {set ::shed(weekdays,4) 1}
    set ::shed(strtype) [getTimeStr $::shed(type)]
    if {![info exists ::blook(svccfgpath)] || [lempty $::blook(svccfgpath)]} {
        set ::blook(svccfgpath) [file normalize [appFile $::blook(svcname).cfg]]
    }
}
proc checkHour {hour} {
    return [expr {[string is integer -strict $hour] && $hour >= 1 && $hour <= 24 || [lempty $hour]}]
}
proc checkHour2 {hour} {
    return [expr {[string is integer -strict $hour] && $hour >= 0 && $hour <= 23 || [lempty $hour]}]
}
proc checkMinutes {min} {
    return [expr {[string is integer -strict $min] && $min >= 0 && $min <= 59 || [lempty $min]}]
}
proc checkDay {day} {
    return [expr {[string is integer -strict $day] && $day >= 1 && $day <= 999 || [lempty $day]}]
}
proc checkWeek {week} {
    return [expr {[string is integer -strict $week] && $week >= 1 && $week <= 7  || [lempty $week]}]
}
proc addEntries {} {
    if {![array exists ::cfgetab]} {return}
    set names [array names ::cfgetab vals,*]
    foreach name $names {
        lappend ents [lindex $::cfgetab($name) 0] [lindex $::cfgetab($name) 1] [lindex $::cfgetab($name) 2]
    }
    array unset ::cfgetab
    foreach {org login password} $ents {
        entryAdd $org $login $password
    }
}
proc encrypt {msg} {
    return [aes::aes -hex -dir encrypt -key $::blook(key) $msg]
}
proc decrypt {msg} {
    return [aes::aes -dir decrypt -key $::blook(key) [binary format H* $msg]]
}
proc chooseFile {} {
    set types {
        {"Text files" {.txt} TEXT}
        {"All files"  {.*}   TEXT}
    }
    set fileName [tk_getSaveFile -filetypes $types -parent .\
            -initialfile $::blook(svccfgpath)\
            -title " "\
            -defaultextension .cfg]
    if {![lempty $fileName]} {
        set ::blook(svccfgpath) $fileName
    }
}
proc saveSettings {{wd 1}} {
    setDefaults
    if {[catch {
            set f [open $::blook(svccfgpath) w]
        } desc]} {
        alertError "    \n$::blook(svccfgpath)\n$desc\n       \n    ."
        return
    }
    foreach idx {sync auto type hour timeh timem day week} {
        puts $f "set ::shed($idx) $::shed($idx)"
    }
    loop i 0 7 {
        puts $f "set ::shed(weekdays,$i) $::shed(weekdays,$i)"
    }
    puts $f "set ::shed(savetime) [clock seconds]"
    if {[array exists ::etab]} {
        set names [array names ::etab vals,*]
        foreach name $names {
            puts $f "set ::cfgetab($name) {$::etab($name)}"
        }
    }
    close $f
    updateInit
    if {$wd} {wm state . withdrawn}
    createService
}
proc updateInit {} {
    if {[catch {
            set f  [open $::blook(inipath) r]
            set ft [open $::blook(inipath).tmp w]
        } desc]} {
        alertError "    \n$::blook(inipath)\n$desc\n       \n    ."
        return
    }
    close $f
    for_file line $::blook(inipath) {
        if {[lindex $line 1] ne "::blook(svccfgpath)"} {
            puts $ft $line
        }
    }
    puts $ft "set ::blook(svccfgpath) {$::blook(svccfgpath)}"
    close $ft
    file rename -force $::blook(inipath).tmp $::blook(inipath)
}
proc stateName {state} {
    switch -- $state {
        "notexists"     {return "  "}
        "stopped"       {return " "}
        "running"       {return " "}
        "stop_pending"  {return " ..."}
        "start_pending" {return " ..."}
        default            {}
    }
}
proc startEventLoop {} {
    while {1} {
        set state [serviceStatus]
        set text  [stateName $state]
        if {[isPaused]} {set text "$text ()"}
        winico taskbar modify [winico info] -text $text
        after $::blook(uto) {set ::mnext 1}
        vwait ::mnext
    }
}
proc startServer {} {
#    if {[catch {comm::comm config -listen 1 -local 1 -port $::blook(mgrport) -silent 0} desc]} {
#        alertError "        .\n$desc"
#        doExit
#    }
    if {[catch {socket -server newConn -myaddr 127.0.0.1 $::blook(mgrport)} desc]} {
        alertError "        .\n$desc"
        doExit
    }
    return
}
proc newConn {channel clientAddress clientPort} {
    if {(($clientAddress ne "127.0.0.1") && ($clientAddress ne "localhost"))} {
        return
    }
    readFromChannel $channel
    return
}
proc readFromChannel {ch} {
    if {[eof $ch]} {
        close $ch
        return
    }
    catch {
        gets $ch msg
        handleSvcMessage $msg
    }
    return
}
proc sendMsg {msg} {
    #comm::comm send -async $::blook(svcport) handleMgrMessage "{$msg}"
    if {[catch {set ch [socket 127.0.0.1 $::blook(svcport)]} desc]} {
        #alertError "   .\n$desc"
        return
    }
    puts $ch $msg
    flush $ch
    close $ch
    return
}
proc handleSvcMessage {msg} {
    if {[lempty $msg]} {return}
    switch -- $msg {
        "paused"    {set ::blook(paused) 1}
        "notpaused" {set ::blook(paused) 0}
        default     {alertWarning "   : $msg"}
    }
    return
}

loadPackages {Tcl 8.4 Tk 8.4 Tclx 8.4 Winico 0.61 registry 1.1.3 md5 2.0.0 aes 1.0.0}
source net.tcl
package require tile
namespace import -force ttk::*
initManager
