#!/usr/bin/wish

# Copyright (c) 2002-2014, Botik Technologies, Ltd.
# See license.txt for more information.

# ================= Main ========================================================
#
# 
#
#     proc startMainWindow { }
#     proc changeStatus { uColorName } -- uColorName:   {RED YELLOW GREEN}
#     proc resetIcons { }
#
# 
#
#     botikkey
#     myImages
#
#   
#

proc startMainWindow { } {
   global botikkey myImages iface
   if {[windowsPlatform]} {
      set icofile [file join . img botik_key.ico]
      wm iconbitmap . -default $icofile
   }
   wm resizable . 0 0
   wm protocol . WM_DELETE_WINDOW {doExit}
   createIcon
   setWindowIcon .
   catch {destroy .mw}
   labelframe .mw -text $botikkey(version) -font $iface(font2)
   pack  .mw -side left -fill both
   focus -force .mw
   frame .mw.top
   frame .mw.top.left
   frame .mw.top.right
   frame .mw.bottom
   pack  .mw.top       -side top   -padx 0 -pady 0 -anchor s -fill y
   pack  .mw.top.left  -side left  -padx 3 -pady 0 -anchor w -fill y
   pack  .mw.top.right -side right -padx 3 -pady 3 -anchor e -fill both
   pack  .mw.bottom    -side top   -padx 3 -pady 0 -anchor n -fill y
   label .mw.top.left.lbl -image $myImages(YELLOW) -bd 1 -relief ridge
   pack .mw.top.left.lbl -side left -padx 3
   setImgButtons .mw.top.right 2 2 top n [subst { 
         help  $myImages(helpNormal)  {showHelp HelpForMainWindow}
         setup $myImages(setupNormal) runOptions
         log   $myImages(logNormal)   showLogWindow
   } ]
   label .mw.bottom.ll -text ":"
   tk_optionMenu .mw.bottom.region botikkey(region_by_user) WORLD LOCAL NONE
   pack .mw.bottom.ll .mw.bottom.region -side left -anchor w
   frame .mw.balance
   frame .mw.tariff
   pack .mw.balance -side top -padx 0 -pady 0 -anchor s -fill y
   pack .mw.tariff  -side top -padx 0 -pady 0 -anchor s -fill y
   label .mw.balance.ll -text ": "
   label .mw.balance.lm -textvar botikkey(balance)
   label .mw.balance.lr -text " ."
   pack .mw.balance.ll .mw.balance.lm .mw.balance.lr -side left -padx 0
   label .mw.tariff.ll -text ": "
   label .mw.tariff.lm -textvar botikkey(tariff)
   pack .mw.tariff.ll .mw.tariff.lm -side left -padx 0
   trace add variable botikkey(region_by_user) {write} newAccessRegion
   #   ,      :
   if {[windowsPlatform] && !$::notray} {
      bind . <Unmap> "wm state . withdrawn"
      #     Esc,   BotikKey :
      bind . <Escape> [bind . <Unmap>]
   }
   draw_battery_frame
   ::tk::PlaceWindow .
   setTitlesAndLights YELLOW "Init ::    \"$::net(name)\" -- [strTime]"  "Init::Botik Key"
   changesInHelpWindow HelpForMainWindow
   return
}
proc newAccessRegion {n1 n2 op} {
   newAccessRegion2 $::botikkey(region_by_user)
}
proc newAccessRegion2 {region} {
   global botikkey
   if {$botikkey(region_request) eq $region} {return}
   set botikkey(region_request) $region
   set botikkey(region_by_user) $region
   writeLog 0 GRAY "[strTime]   ,   $region"
   if {$botikkey(gbk)} {
      set reg [string tolower $region]
      if {[lsearch {world local none} $reg] == -1} {return}
      changeStatus YELLOW
      sendToGbk "set_status_to\n$reg [clock seconds]"
      return
   }
   if {$botikkey(client_started)} {
      set botikkey(client_started) 0   
      restartClient
   }
   return
}
proc isBatteryTariff {} {
    return [expr {$::botikkey(timequota) > 0}]
}
proc draw_battery_frame {} {
    global botikkey
    if {![isBatteryTariff]} {
        catch {destroy $::botikkey(batframe)}
        return
    }
    if {![winfo exists $::botikkey(batframe)]} {
        set tariff " \"\""
        if {[string match "**" $::botikkey(tariff)]} {
            set tariff " \"$::botikkey(tariff)\""
        }
        labelframe $::botikkey(batframe) -text $tariff -font {Arial 13 bold} -width 210 -height 200
        pack $::botikkey(batframe) -side right -fill both
        battery_skin
    } else {
        redraw_battery_skin
    }
}
proc imgExists {name} {
    return [lcontain [image names] $name]
}
# pc = remain frac. (from 0 to 1.0)
proc draw_battery {c pc rtime} {
    if {![winfo exists $::botikkey(batframe)]} {return}
    if {![string is double -strict $pc]} {return}
    $c delete all
    if {$pc < 0} {set pc 0}
    if {$pc > 1} {set pc 1}
    set pcd [expr {int($pc*100)}]%
    set wid [$c cget -width]
    set hei [$c cget -height]
    set tag "batimage"
    if {![imgExists $tag]} {
        # load green battery from file
        image create photo $tag -file [file join img battery.gif]
    }
    $c create image 0 0 -image $tag -anchor nw -tags $tag -state disabled
    if {$pc < 0.25} {set tfill "red"} else {set tfill "black"}
    set rtext ":\n[mins_to_text $rtime]"
    $c create text [expr {int($wid/2.0)}] [expr {int($hei/2.0)}] -fill $tfill -text $rtext -tags remain -font {Arial 18 bold} -anchor center
    set ::botikkey(battery_rtime) $rtime
    set iw   [image width $tag]
    set ih   [image height $tag]
    set ry   $ih
    set minx 10
    set maxx 185
    set diff [expr {$maxx - $minx}]
    set rx [expr {int($minx + $diff*$pc)}]
    set rl [expr {int($maxx - $diff*$pc)}]
    if {$rx < $minx} {set rx $minx}
    if {$rx > $maxx} {set rx $maxx}
    if {$rx+$rl > $maxx} {
        set rl [expr {$maxx-$rx}]
    }
    $c create rect $rx 0 [expr {int($rx+$rl)}] $ry -fill gray -tags plot -width 0
    $c raise $tag
    $c raise plot
    $c raise remain
    update
}
proc mins_to_text {mins} {
    set chours [expr {int(floor($mins/60.0))}]
    set cmins [expr {$mins % 60}]
    set ctext ""
    if {$chours > 0} {append ctext "$chours . "}
    append ctext "$cmins ."
    return $ctext
}
proc redraw_battery_skin {} {
    set ttime $::botikkey(timequota)
    if {$ttime <= 0 || $::botikkey(region_by_user) ne "WORLD"} {return}
    set ctime [expr {$::botikkey(timeused) + ([clock seconds] - $::botikkey(time_recv))/60}]
    if {$::botikkey(region_by_user) ne "WORLD"} {
        set ctime $::botikkey(timeused)
    }
    set rtime [expr {$ttime - $ctime}]
    if {$::botikkey(timeused) > $::botikkey(timequota)} {
        set ctime 0
        set rtime 0
    }
    set pc [expr {double($rtime)/double($ttime)}]
    set cpc [expr {double($ctime)/double($ttime)}]
    set w $::botikkey(batframe)
    if {$rtime == $::botikkey(battery_rtime)} {return}
    draw_battery .bat.c $pc $rtime
    set pcd [expr {int($cpc*100)}]%
    set ctext ": [mins_to_text $ctime] ($pcd)"
    set w $::botikkey(batframe)
    $w.cl configure -text $ctext
}
proc battery_skin {} {
    # ttime = total tariff time (in minutes)
    # ctime = time consumed (in minutes)
    # rtime = remaining tariff time (in minutes)
    set ttime $::botikkey(timequota)
    if {$ttime <= 0} {return}
    set ctime [expr {$::botikkey(timeused) + ([clock seconds] - $::botikkey(time_recv))/60}]
    if {$::botikkey(region_by_user) ne "WORLD"} {
        set ctime $::botikkey(timeused)
    }
    set rtime [expr {$ttime - $ctime}]
    set pc [expr {double($rtime)/double($ttime)}]
    set cpc [expr {double($ctime)/double($ttime)}]
    set w $::botikkey(batframe)
    catch {destroy $w.c}
    canvas $w.c -width 200 -height 90
    pack $w.c -fill both -padx 3 -pady 3 -side top
    draw_battery $w.c $pc $rtime
    set pcd [expr {int($cpc*100)}]%
    set ctext ": [mins_to_text $ctime] ($pcd)"
    catch {destroy $w.cl}
    label $w.cl -text $ctext -font {Arial 11} -anchor w
    pack $w.cl -side top -padx 4 -pady 2 -fill x
    if {$::botikkey(region_by_user) eq "WORLD"} {
        set butext [batActionName 0]
    } else {
        set butext [batActionName 1]
    }
    catch {destroy $w.bf.action}
    frame $w.bf
    pack $w.bf -fill x
    button $w.bf.action -text $butext -font {Arial 12 bold} -command setBatRegion
    pack $w.bf.action -side left -padx 2 -pady 4
    catch {destroy $w.time}
    frame $w.time
    pack $w.time -fill both -expand 1
    frame $w.time.l1f
    pack $w.time.l1f -side top -fill both -expand 1
    frame $w.time.om
    pack $w.time.om -side left -fill both -expand 1
    frame $w.time.l2f
    pack $w.time.l2f -side left -fill both -expand 1
    label $w.time.l1f.l1 -text "  :" -font {Arial 12}
    tk_optionMenu $w.time.om.menu ::botikkey(stoptime) "5 " "10 " "15 " "20 " "30 " "45 " "60 " ""
    label $w.time.l2f.l2 -text "" -font {Arial 12}
    pack $w.time.l1f.l1 $w.time.om.menu $w.time.l2f.l2 -side left -padx 1 -pady 1
    $w.time.om.menu configure -font {Arial 12}
    trace add variable ::botikkey(stoptime) {write} newStopTime
}
proc newStopTime {args} {
    set w $::botikkey(batframe)
    if {$::botikkey(stoptime) eq ""} {
        $w.time.l1f.l1 configure -text "   :"
        $w.time.l2f.l2 configure -text ""
    } else {
        $w.time.l1f.l1 configure -text "  :"
        $w.time.l2f.l2 configure -text ""
    }
}
proc updateBatRegion {} {
    set w $::botikkey(batframe)
    catch {
    if {$::botikkey(region_by_user) eq "WORLD"} {
        $w.bf.action configure -text [batActionName 0]
    } else {
        $w.bf.action configure -text [batActionName 1]
    }}
}
proc batActionName {n} {
    if {$n == 0} {
        return " "
    } else {
        return " "
    }
}
proc setBatRegion {} {
    set w $::botikkey(batframe)
    if {$::botikkey(region_by_user) eq "WORLD"} {
        catch { $w.bf.action configure -text [batActionName 1] }
        setRegionTo "NONE"
    } else {
        catch { $w.bf.action configure -text [batActionName 0] }
        setRegionTo "WORLD"
    }
}
proc resetIcons { } {
   setIconsAndLights $::botikkey(currentUColorName)
}
#   ,    {RED YELLOW GREEN GREEN_LOCAL}
proc checkColorName { colorName } {
   global botikkey
   if { [set i [lsearch -exact { RED YELLOW GREEN GREEN_LOCAL } $colorName]] < 0 } {
      return YELLOW
   }
   if {($i == 2) && ($botikkey(region_by_server) eq "LOCAL")} {
      return GREEN_LOCAL
   }
   return $colorName
}

proc setIconsAndLights {uColorName} {
   global botikkey
   set colorName [checkColorName $uColorName] ; # RED | YELLOW | GREEN | GREEN_LOCAL 
   set newIcon   [lindex $botikkey($colorName) [validateIconType]]
   configImage $colorName
   if {$botikkey(currentIcon) ne $newIcon} {
      set botikkey(currentUColorName) $colorName
      set botikkey(currentIcon)       $newIcon
      trayIcon modify
   }
   #  BotikKey   ,      :
   #  , .     .
   foreach w {. .wBkLog .wHelp .wBkOptions} {
       if {[winfo exists $w] && $w ne "."} {
           if {![winfo ismapped $w]} {
              wm deiconify $w
              raise $w
           }
           setWindowIcon $w
       } elseif {$w eq "." && [winfo ismapped $w]} {
           setWindowIcon $w
       }
   }
   return
}

proc configImage {colorName} {
   global myImages
   .mw.top.left.lbl configure -image $myImages($colorName)
}

proc setTitlesAndLights {uColorName t ti} {
   setIconsAndLights $uColorName
   set ::roottitles [list $t $ti]
   if {[winfo ismapped .]} {
       wm title . $t
       wm iconname . $ti
   }
   update
}
proc initMenu {} {
      global botikkey
      catch {destroy .m}
      update
      if {[winfo exists .m]} {return}
      if {[catch {menu .m -tearoff 0}]} {return}
      .m add command -label "" -command "showHelp HelpForMainWindow"
      .m add command -label "" -command "runOptions"
      .m add command -label "" -command "showLogWindow"
      .m add separator
      foreach {region name} $botikkey(modenames) {
          if {$region ne $botikkey(region_by_server)} {
          .m add command -label $name -command "setRegionTo $region"
        }
      }
      .m add separator
      if {![windowsPlatform]} {
          .m add command -label " " -command "catch {destroy .m}"
          .m add separator
      }
      .m add command -label "" -command "doExit"
}
proc setRegionTo {region} {
    set ::botikkey(region_by_user) $region
}
proc displayMainWindow {} {
    wm deiconify .
    raise .
    focus -force .
    setWindowIcon .
    if {[info exists ::roottitles]} {
        set t  [lindex $::roottitles 0]
        set ti [lindex $::roottitles 1]
        wm title . $t
        wm iconname . $ti
        update
    }
    draw_battery_frame
}
proc tray_cmd {winmsg x y} {
    if {$winmsg eq "WM_LBUTTONDBLCLK"} {
        #set ::botikkey(region_by_user) [expr {($::botikkey(region_by_user) eq "WORLD") ? "LOCAL" : "WORLD"}]
    } elseif {$winmsg eq "WM_LBUTTONUP"} {
        #      :   ,   ;
        #  ,  .
        if {[winfo ismapped .]} {
            wm state . withdrawn
            #dolog "  ."
            return
        }
        displayMainWindow
    } elseif {$winmsg eq "WM_RBUTTONUP"} {
        #      :  .
        initMenu
        .m post $x $y
        #.m activate 0
    }
}
proc warning {} {
   if {$::botikkey(gbk)} {return "ok"}
   set msg "   BotikKey.   \n   (    BotikKey)."
   set ans [tk_messageBox -type okcancel -icon warning -title "" -parent .mw -message $msg]
   return $ans
}
proc delete_icons {} {
    global botikkey
    if {$::notray || ![windowsPlatform]} {return}
    loop i 1 $botikkey(notifcn,limit) {
        catch {winico delete $botikkey(notifcn$i,icon)}
    }
    winico delete $botikkey(mainicon)
    update
}
proc doExit {{ask 1}} {
    if {$ask} {
        set ans [warning]
    } else {
        set ans "ok"
    }
    if {$ans eq "ok"} {
        delete_icons; exit
    }
}
proc changeStatus { uColorName } {
    global botikkey
    set cName [checkColorName $uColorName]
    switch $cName {
        RED    {
            setTitlesAndLights RED "NONE!    \"$::net(name)\" -- [strTime]"  "NONE::Botik Key"
        }
        YELLOW {
            setTitlesAndLights YELLOW ".    \"$::net(name)\" -- [strTime]" "Waiting::Botik Key"
        }
        GREEN - GREEN_LOCAL {
            set region $botikkey(region_by_server)
            setTitlesAndLights GREEN  "$region!    \"$::net(name)\" -- [strTime]" "${region}::Botik Key"
            if {[clock seconds] - $botikkey(chk_version_time) >= 12*3600} {set botikkey(chk_version) 1}
            if {$botikkey(chk_version)} {
                readURL $botikkey(url_chk_version) 10000
                set botikkey(chk_version)      0
                set botikkey(chk_version_time) [clock seconds]
            }
        }
    }
    return
}
proc findTraybarIconPos {} {
    global botikkey positions
    if {$::notray} {return}
    set ico         $botikkey(mainicon)
    set info        {}
    catch {set info [winico info $ico]}
    if {$info eq {}} {return -1}
    set screendepth [winfo screendepth .]
    set size        "16x16"
    set pos         -1
    set bigdepth    0
    for {set j [expr 9*$positions($botikkey(currentIcon))]} {$j < [expr 9*(1+$positions($botikkey(currentIcon)))]} {incr j} {
        #     9 ,          
        set i [lindex $info $j]
        array set opts $i
        if {![info exists opts(-bpp)] || ![info exists opts(-pos)] || ![info exists opts(-geometry)]} {return -1}
        set depth    $opts(-bpp)
        set pos      $opts(-pos)
        set geometry $opts(-geometry)
        if {$geometry eq "$size" && $depth <= $screendepth} {
            if {$depth > $bigdepth} {
                set pos      $pos
                set bigdepth $depth
            }
        }
    }
    return $pos
}
proc findTaskbarIconsPos {} {
    global botikkey positions
    if {$::notray} {return {0 0}}
    if {[windowsPlatform]} {
        set ico $botikkey(mainicon)
    } else {
        set ico ""
    }
    set info        {}
    catch {set info [winico info $ico]}
    if {$info eq {}} {return [list -1 -1]}
    set screendepth [winfo screendepth .]
    set smallsize   "16x16"
    set bigsize     "32x32"
    set smallpos    -1
    set bigpos      -1
    set smalldepth  0
    set bigdepth    0
    for {set j [expr 9*$positions($botikkey(currentIcon))]} {$j < [expr 9*(1+$positions($botikkey(currentIcon)))]} {incr j} {
        #     9 ,          
        set i [lindex $info $j]
        array set opts $i
        if {![info exists opts(-bpp)] || ![info exists opts(-pos)] || ![info exists opts(-geometry)]} {return [list -1 -1]}
        set depth    $opts(-bpp)
        set pos      $opts(-pos)
        set geometry $opts(-geometry)
        if {$geometry eq "$bigsize" && $depth <= $screendepth} {
            if {$depth > $bigdepth} {
                set bigpos   $pos
                set bigdepth $depth
            }
        } elseif {$geometry eq "$smallsize" && $depth <= $screendepth} {
            if {$depth > $smalldepth} {
                set smallpos   $pos
                set smalldepth $depth
            }
        }
    }
    return [list $smallpos $bigpos]
}
proc setWindowIcon {w} {
    #    :
    if {[winfo exists $w] && [winfo ismapped $w]} {
        update
    }
    foreach {smallpos bigpos} [findTaskbarIconsPos] {}
    if {!$::notray && [winfo exists $w] && [winfo ismapped $w]} {
        if {$smallpos >= 0} {
            catch {winico setwindow $w $::botikkey(mainicon) "small" $smallpos}
        }
    }
    if {!$::notray && [winfo exists $w] && [winfo ismapped $w]} {
        if {$bigpos >= 0} {
            catch {winico setwindow $w $::botikkey(mainicon) "big" $bigpos}
        }
    }
    if {[winfo exists $w] && [winfo ismapped $w]} {
        update
    }
}
proc createIcon {} {
    if {$::notray} {return}
    set ::botikkey(mainicon) ""
    catch {
        set ::botikkey(mainicon) [winico createfrom [file join . img botik.ico]]
    }
    trayIcon add
}
proc trayIcon {cmd} {
    global botikkey
    if {$::notray} {return}
    if {[windowsPlatform]} {
        set pos 0
        catch {set pos [lindex [findTraybarIconPos] 0]}
        if {$cmd eq "add"} {
            if {$pos >= 0} {
                catch {winico taskbar add $botikkey(mainicon) -pos $pos -callback "tray_cmd %m %x %y" -text $botikkey(name)}
            }
        } else {
            if {$pos >= 0} {
                catch {winico taskbar modify $botikkey(mainicon) -pos $pos -text [getMainIconText]}
            }
        }
        setMainIconText
    } else {
        if {$cmd eq "add"} {
            image create photo ico -file $botikkey(currentIcon)
            tktray::icon $botikkey(iconwindow) -image ico -visible true
            bind $botikkey(iconwindow) <ButtonRelease-1> "tray_cmd WM_LBUTTONUP %X %Y"
            bind $botikkey(iconwindow) <Double-ButtonPress-1> "tray_cmd WM_LBUTTONDBLCLK %X %Y"
            bind $botikkey(iconwindow) <ButtonRelease-3> "tray_cmd WM_RBUTTONUP %X %Y"
        } else {
            image create photo ico -file $botikkey(currentIcon)
            $botikkey(iconwindow) configure -image ico
         }
    }
}
proc initNotifcnMenu {n} {
    set w .wm$n
    catch {destroy $w}
    update
    if {[winfo exists $w]} {return $w}
    menu $w -tearoff 0
    $w add command -label "" -command "showTooltip $n"
    $w add separator
    $w add command -label "" -command "hideNotifcn $n"
    $w add command -label " " -command "hideAllNotifcns"
    return $w
}
proc notifcn_tray_cmd {message ico x y n} {
    if {$message eq "WM_LBUTTONDBLCLK"} {
        #       
        showTooltip $n
    } elseif {$message eq "WM_LBUTTONUP"} {
        #      
        showTooltip $n
    } elseif {$message eq "WM_RBUTTONUP"} {
        #      :  .
        #  :
        set m [initNotifcnMenu $n]
        $m post $x $y
    }
    #puts stderr "notifcn_tray_cmd with $message,ico:$ico,x:$x,y:$y,n:$n"
    return
}
proc hideAllNotifcns {} {
    loop n 1 $::botikkey(notifcn,limit) {
        hideNotifcn $n
    }
    return
}
proc hideNotifcn {n} {
    global botikkey
    catch {winico delete $botikkey(notifcn$n,icon)}
    set botikkey(notifcn$n,icon) ""
}
#   
proc showTooltip {n} {
    global botikkey
    winico showtooltip $botikkey(notifcn$n,icon) "  BotikKey" $botikkey(notifcn$n); #max 255 chars
    winico text $botikkey(notifcn$n,icon) $botikkey(notifcn$n,short); #max 127 chars
    if {$::tcl_platform(osVersion) < 5.0} {
        # Win98/ME: just show dialog window
        displayInfo $botikkey(notifcn$n)
    }
    return
}
#show notifications
proc showNotifcn {n} {
    global botikkey
    if {$botikkey(notifcn$n,show)} {return 0}
    if {$n <= 0 || $n >= $botikkey(notifcn,limit)} {return 0}
    if {$botikkey(notifcn$n,icon) ne ""} {return 0}
    if {$::notray || ![windowsPlatform]} {
        displayInfo $botikkey(notifcn$n)
        set botikkey(notifcn$n,show) 1
        return 1
    }
    set botikkey(notifcn$n,icon) [winico load warning]
    winico taskbar add $botikkey(notifcn$n,icon) -callback "notifcn_tray_cmd %m %i %x %y $n"
    set botikkey(notifcn$n,show) 1
    showTooltip $n
    return 1
}
