#!/usr/bin/wish

# Copyright (c) 2003-2014, Botik Technologies, Ltd.
# See license.txt for more information.

# BotikKey -        "".
#   : 19  2003 .

package require Tcl 8.2
package require md5 2.0.0

# ===========================================================================================
proc mkErrorCode { rc {delim "\n"}} {
   global errorCode
   set x $errorCode
   set t [lindex $x 0]
   switch $t {
      ARITH       -
      CHILDKILLED -
      CHILDSUSP   - 
      POSIX       { return "[lindex $x end]$delim$rc" }
      default     { return $rc }
   }
}

proc loadModule { fname } {
   global myModules
   if [info exists myModules($fname)] {
      tk_messageBox -icon warning -type ok -message " \"$fname\"  "
      return
   }
   set ok 0
   catch {
      source $fname
      set ok 1
      set myModules($fname) 1
   } rc
   if { ! $ok } {
      tk_messageBox -icon error -type ok -message \
            "     \"$fname\"\nError: [mkErrorCode $rc]\
            \n\n    .\n\
               ."
      exit
   }
   return
}

proc loadImage {name fname {hint {}}} {
   global myImages hints
   if [info exists myImages($name)] {
      return
   }
   set ok 0
   catch {
      set myImages($name) [image create photo -file $fname]
      set ok 1
   } rc
   if { ! $ok } {
      tk_messageBox -icon error -type ok -message \
            "      : \"$fname\"\nError: [mkErrorCode $rc]\n\n\
                .\
               ."
      exit
   }
   if {![lempty $hint]} {set hints($myImages($name)) $hint}
   return
}

proc debug { str } {
   message info "DEBUG $str"
}

# ===========================================================================================

loadModule bkInit.tcl
loadModule botikUts.tcl
loadModule net.tcl
set ::notray 0
if {[catch {
if {[windowsPlatform]} {
package require Winico 0.61
catch {package require twapi}
} else {
package require tktray 1.1
}
}]} {
    set ::notray 1
}
if {[windowsPlatform]} {
    catch {package require informer 1.0}
}
loadModule botikHlp.tcl
loadModule bkMWin.tcl
loadModule bkLog.tcl
loadModule bkClient.tcl
loadModule bkOptions.tcl
loadModule bkHelp.tcl
loadModule report.tcl

loadImage  helpNormal        img/help.gif       ""
loadImage  logNormal         img/logfile.gif    ""
loadImage  setupNormal       img/setup.gif      ""
loadImage  saveNormal        img/save_as.gif    " ..."
loadImage  closeNormal       img/stop.gif       "/"
loadImage  fnameNormal       img/pswd_write.gif "    "
loadImage  readPswd          img/pswd_read.gif  "  ,   "
loadImage  yesNormal         img/yes.gif        " "
loadImage  yesAndWriteNormal img/yes_write.gif  "   "
loadImage  noNormal          img/no.gif         " "
loadImage  prevNormal        img/prev.gif       "   "
loadImage  nextNormal        img/next.gif       "   "
loadImage  refreshNormal     img/refresh.gif    "  "
loadImage  fig1              img/fig1.gif
loadImage  YELLOW            img/YELLOW.gif
loadImage  GREEN             img/GREEN.gif
loadImage  GREEN_LOCAL       img/GREEN_LOCAL.gif
loadImage  RED               img/RED.gif

# =================   =============================================

initValues
initHelp
startMainWindow
checkSingleInvocationOfBotikKey

#  ,     ,   
#  (. setVarsOptions).  :      
#     ..:
readIniFile
clientMainLoop
