#!/usr/bin/wish

# Copyright (c) 2002-2014, Botik Technologies, Ltd.
# See license.txt for more information.

# ,    , 
#     "-" (http://www.botik.ru).
#   BotikTools        .

#  :
set ::net(name) ""

#  :
set ::net(domain) botik.ru

#   ""   email-;
#    http://www.botik.ru/proxy.config
proc isLocal {host ip} {
    return [expr {[regexp {\.botik\.ru$} $host]
        || [regexp {\@botik\.ru$} $host]
        || [regexp {\.pereslavl\.ru$} $host]
        || [regexp {\@pereslavl\.ru$} $host]
        || [regexp {\.botik\.yaroslavl\.su$} $host]
        || [regexp {\@botik\.yaroslavl\.su$} $host]
        || [regexp {^193\.232\.174\.\d+$} $ip]
        || [regexp {^95\.129\.13[6-9]\.\d+$} $ip]
        || [regexp {^217\.15\.133\.4[0-7]$} $ip]
        || [regexp {^217\.15\.139\.12[89]$} $ip]
        || [regexp {^217\.15\.139\.13[0-5]$} $ip]
        || [regexp {^192\.168\.\d+\.\d+$} $ip]
        || [regexp {^127\.0\.0\.\d+$} $ip]}]
}
proc clockFormat {fmt {time ""}} {
    if {![llength $time]} {set time [clock seconds]}
    set cmd "clock format $time -format \"$fmt\""
    if {$::tcl_version >= 8.5} {append cmd " -timezone +0400"}
    return [eval $cmd]
}
proc http_get {url} {
    package require http
    if {[string match https:* $url]} {
        # if secure http (https)
        package require tls
        http::register https 443 tls::socket
    }
    if {[info exists ::env(TMP)]} {
        set tmp $::env(TMP)
    } elseif {[info exists ::env(TEMP)]} {
        set tmp $::env(TEMP)
    } elseif {[info exists ::env(APPDATA)]} {
        set tmp [file join $::env(APPDATA) BotikTools]
    } else {set tmp "~"}
    set fpath [file join $tmp [file tail $url]]
    set fd [open $fpath w]
    fconfigure $fd -translation binary
    catch {
        set ok 0
        set token [http::geturl $url -channel $fd -binary 1 -timeout 40000]
        set ok 1
    }
    catch {close $fd}
    set ncode 200
    if {$ok} {
        upvar #0 $token state
        set ncode [http::ncode $token]
    }
    if {!$ok || $ncode != 200} {
        return "0"
    } else {
        return $fpath
    }
}
proc splash_window {} {
    package require Img
    set w .splash
    catch {destroy $w}
    toplevel $w
    wm overrideredirect $w 1
    set c $w.c
    set width 360
    set height 80
    set img [file join . img btools-logo.gif]
    canvas $c -width $width -height $height -border 10
    image create photo splash -file $img
    $c create image 0 10 -anchor nw -image splash
    $c create text 100 25 \
        -text "  BotikTools.\n, ..." \
        -font {Arial 13 bold} -anchor nw
    set wscreen [winfo screenwidth $w]
    set hscreen [winfo screenheight $w]
    set x [expr {($wscreen - $width) / 2}]
    set y [expr {($hscreen - $height) / 2}]
    wm geometry $w +$x+$y
    pack $w.c
    raise $w
    focus -force $w
    update
    return $w
}
proc add2Version {str varname} {
    if {![regexp {\*$} [set $varname]] && ![regexp -- {-$} [set $varname]]} {
        catch {
        append $varname $str
        wm title . [set $varname]}
    }
}
proc selfUpdate {progname oldver newver btver} {
    set fpath "0"
    set msg ""
    if {[windowsPlatform] && ![info exists ::wine_used]} {
        # BotikTools for Windows distribution is used,
        # so do the auto-update
        set url "http://www.$::net(domain)/~botik/download/windows/BotikTools.v.$btver.exe"
        catch {set w [splash_window]}
        catch {
            # download installer, return filepath, or "0" if error
            set fpath [http_get $url]
            catch {destroy $w}
            if {$fpath ne "0"} {
                # silent install
                exec $::env(COMSPEC) /c start $fpath /S
                return "no"
            }
        }
        # this line is unreachable, unless something went wrong with auto-update,
        # e.g. when user does not have admin privileges on Windows PC;
        # fallback to alert for manual install:
        set msg " (,     )"
    }
    set answer [alertQuestion "   : $progname v.$oldver.\
             BotikTools   : $progname v.$newver.\
                  BotikTools$msg.\
           \n\n       ?"]
    return $answer
}
