# profiler
# see BotikMap.tcl for usage example
proc debugprof {str} {
    catch {
        set f [open prof.txt a]
        puts $f $str
        close $f
    }
}

namespace eval ::profiler {
    variable cnt
    variable sum
    variable min
    variable max
    variable cmdlist {}
    variable tmstart 0
    variable tmend 0
    variable tmp
    variable started 0

    proc init {} {
        variable db
        variable cnt
        variable sum
        variable min
        variable max
        variable tmp
        array unset cnt
        array unset sum
        array unset min
        array unset max
        array unset tmp
        array unset db
    }
    proc stepping {names} {
        foreach name $names {
            trace add exec $name enterstep ::profiler::enterStep
            trace add exec $name leavestep ::profiler::leaveStep
        }
    }
    proc noStepping {names} {
        foreach name $names {
            trace remove exec $name enterstep ::profiler::enterStep
            trace remove exec $name leavestep ::profiler::leaveStep
        }
    }
    proc enterStep {cmd _} {
        debugprof "enter:$cmd"
    }
    proc leaveStep {cmd code result _} {
        debugprof "leave:$cmd"
    }
    proc add {cmdlist_a} {
        variable cmdlist
        set cmdlist [concat $cmdlist $cmdlist_a]
        set cmdlist [lsort -unique $cmdlist]
    }
    proc enter {cmd op} {
        variable db
        set cmd [lindex $cmd 0]
        if {![info exists db($cmd)]} {
            set db($cmd)    ""
            set db($cmd,n)  0
            set db($cmd,tt) 0
        }
        incr db($cmd,n)
        set db($cmd,t) [clock milliseconds]
    }
    proc leave {cmd code result op} {
        variable db
        set cmd [lindex $cmd 0]
        incr db($cmd,tt) [expr {[clock milliseconds] - $db($cmd,t)}]
    }
    proc goodcmd {cmd} {
        if {[string match "*profiler*" $cmd] || $cmd eq "debug" || 
            [string match "*clock*" $cmd] || [string match "*bgerror*" $cmd]} {return 0}
        return 1
    }
    proc begin {args} {
        variable cmdlist
        variable tmstart
        variable started
        foreach cmd $cmdlist {
            if {![goodcmd $cmd]} {continue}
            catch {
                trace add execution $cmd enter ::profiler::enter
                trace add execution $cmd leave ::profiler::leave
            }
        }
        set tmstart [clock milliseconds]
        set started 1
    }
    proc end {args} {
        variable cmdlist
        variable tmstart
        variable tmend
        variable db
        variable cnt
        variable sum
        variable min
        variable max
        variable started
        set started 0
        set tmend [clock milliseconds]
        foreach cmd $cmdlist {
            if {![goodcmd $cmd]} {continue}
            catch {
                trace remove execution $cmd enter ::profiler::enter
                trace remove execution $cmd leave ::profiler::leave
            }
        }
        debugprof "--------- PROFILER STATS ---------------------------------------------"
        array set tmp {}
        array set cnt {}
        array set sum {}
        array set min {}
        array set max {}
        set totaltime 0
        set cmds {}
        foreach cmd $cmdlist {
            if {![info exists db($cmd,tt)]} {continue}
            if {![info exists cnt($cmd)]} {set cnt($cmd) $db($cmd,n)}
            if {![info exists sum($cmd)]} {set sum($cmd) $db($cmd,tt)}
            if {![info exists min($cmd)]} {set min($cmd) 0}
            if {![info exists max($cmd)]} {set max($cmd) 0}
            set delta $db($cmd,tt)
            incr totaltime $delta
            if {$min($cmd) == 0 || $delta < $min($cmd)} {set min($cmd) $delta}
            if {$max($cmd) == 0 || $delta > $max($cmd)} {set max($cmd) $delta}
            lappend cmds [list $cmd $sum($cmd)]
        }
        debugprof "Total time: [expr {$tmend - $tmstart}] ms ([format %.2f%% [expr {$totaltime*100.0/($tmend-$tmstart)}]] overhead+non-profiled commands)"
        debugprof "command     sum_ms ncalls ms_per_call"
        set cmds [lsort -integer -index 1 -decreasing $cmds]
        foreach cmd0 $cmds {
            foreach {cmd tm} $cmd0 {break}
            if {![info exists cnt($cmd)] || $cnt($cmd)==0} {continue}
            if {![info exists sum($cmd)]} {set sum($cmd) 0}
            if {![info exists min($cmd)]} {set min($cmd) 0}
            if {![info exists max($cmd)]} {set max($cmd) 0}
            set avg [expr {double($sum($cmd))/double($cnt($cmd))}]
            #set percent [expr {$sum($cmd)*100.0/($totaltime)}]
            #debugprof [format "%-12s %d %d %d %d %d %.2f%%" $cmd $min($cmd) $max($cmd) $avg $sum($cmd) $cnt($cmd) $percent]
            debugprof [format "%-12s %d %d %.2f" $cmd $sum($cmd) $cnt($cmd) $avg]
        }
        debugprof "----------------------------------------------------------------------"
        init
    }
}
