#!/usr/bin/wish

# Copyright (c) 2002-2014, Botik Technologies, Ltd.
# See license.txt for more information.

package require Tclx

proc sendmail {body {btrep {0 {}}}} {
    global env whoami_ botik
    if {[catch {
        set ::reportNotSent 1
        set subj  [lindex $btrep 1]
        set btrep [lindex $btrep 0]
        package require mime 1.3.5
        package require smtp 1.3.5
        #       WhoAmI
        _whoAmI
        set myinfo ""
        set myemail ""
        if {[array exists whoami_]} {
            set all [array get whoami_]
            foreach {elt val} $all {
                if {[lempty $elt]} {continue}
                append myinfo "\n:$elt:$val"
                if {$elt eq "Time"} {
                    catch {append myinfo "\n:Client time:$botik(cli_time)"}
                }
            }
        }
        if {[info exists ::conn(login)]} {append myinfo "\n:Login:$::conn(login)"}
        append myinfo "\n[systemInfo]"
        #    
        if {[catch {set from ${env(USERNAME)}-$whoami_(UserName)}]} {
            if {[catch {set from ${env(USERNAME)}-$env(USERDOMAIN)}]} {
                set from "\"BotikTools_user\" <botik-tools@botik.ru>"
            }
        }
        if {[info exists ::wReport(From)] && ![lempty $::wReport(From)]} {
            set from "\"$from\" <$::wReport(From)>"
            set myemail $::wReport(From)
        } elseif {[info exists whoami_(Email)] && ![lempty $whoami_(Email)]} {
            set from "\"$from\" <$whoami_(Email)>"
            set myemail $whoami_(Email)
        }
        if {![llength $myemail]} {
            catch {
                set file [appFile tester.init]
                if {[llength $file]} {
                    set fd [open $file r]
                    gets $fd line
                    close $fd
                    set myemail [lindex [split $line ":"] end-1]
                    if {![string match "*@*" $myemail]} {set myemail ""}
                }
            }
        }
        set from    [tokoi $from]
        set replyto $myemail
        #    
        if {[info exists ::ping(ver)]} {
            set pname "BotikTester v.$::ping(ver)"
        } elseif {[info exists ::botikkey(ver)]} {
            set pname "BotikKey v.$::botikkey(ver)"
        } elseif {[info exists ::plot(ver)]} {
            set pname $::plot(ver)
        } else {
            set pname "BotikTools"
        }
        #  
        if {!$btrep} {
            set subject [tokoi BotikTools\::[lindex [split $body "\n"] 0]]
        } else {
            set subject [tokoi $subj]
        }
        #  
        if {!$btrep} {
            set body [tokoi "$pname\n$myinfo\n\n$body"]
        } else {
            set body [tokoi $body]
        }
        # if not BotikMap and not BotikTester
        if {![info exists ::plot(version)] && !$btrep} {
            append body [tokoi "\n\n :\n\n[getEnv]"]
        }
        #  
        set parts [mime::initialize -canonical "text/plain; charset=koi8-r; format=flowed" -string $body -encoding 8bit]
        if {[array exists ::botikkey] && [file exists $::botikkey(log)]} {
            set file [addToArchive $::botikkey(log)]
            lappend parts [mime::initialize -canonical "application/data; name=\"[file tail $file]\"" -file $file]
        }
        if {0} {
            #[info exists ::plot(version)]
            # dump all BotikMap environment to a file, compress and send it
            set fd [open ~/progenv.txt w]
            puts $fd [getEnv]
            close $fd
            set arch [addToArchive ~/progenv.txt]
            if {[file size $arch] <= 9*1024*1024} {
                lappend parts [mime::initialize -canonical "application/data; name=\"[file tail $arch]\"" -file $arch]
            }
        }
        if {$btrep} {
            set devemails $::ping(repair)
            if {[info exists ::wReport(From)] && ![lempty $::wReport(From)]} {
                lappend devemails $::wReport(From)
            }
            set devemails [join $devemails ,]
        } else {
            set devemails "botik-tools@botik.ru"
        }
        if {[llength $parts] > 1} {
            set token [mime::initialize -canonical "multipart/mixed" -parts $parts]
        } else {
            set token [mime::initialize -canonical "text/plain; charset=koi8-r; format=flowed" -string $body -encoding 8bit]
        }
        smtp::sendmessage $token \
            -originator $from \
            -recipients $devemails \
            -servers    mail.$::net(domain) \
            -header     [list To $devemails] \
            -header     [list Subject $subject] \
            -header     [list From $from] \
            -header     [list Reply-To $replyto]
        mime::finalize $token -subordinates all
    }]} {
        unset -nocomplain ::reportNotSent
        if {$btrep} {
            set ::mail_log    {}
            set ::mail_status 1
            set ::mail_result $::errorInfo
        }
        set msg "   .\n$::errorInfo"
        tk_messageBox -icon error -type ok -title "" -message "$msg"
    } else {
        unset -nocomplain ::reportNotSent
        if {$btrep} {
            set ::mail_log    {}
            set ::mail_status 0
            set ::mail_result {}
        } else {
            set msg "  ."
            tk_messageBox -icon info -type ok -title "" -message "$msg"
        }
    }
}
proc addToArchive {file} {
    if {[info exists ::bt_exe] && $::bt_exe} {
        catch {bzip2 -zfk $file}
    }
    if {[file exists $file.bz2]} {
        return $file.bz2
    }
    set arfile [file normalize [file join ~ $file.tar.bz2]]
    catch {
        exec ../bin/tar.exe -cjf "$arfile" "$file"
    }
    if {[file exists $arfile]} {
        return $arfile
    }
    return $file
}
proc asksendmail {body} {
    if {[lempty $body]} {return}
    set prog " "
    if {[info exists ::botiktools(progname)]} {
        append prog " " $::botiktools(progname)
    } else {
        append prog "  Botik Tools"
    }
    set msg "$prog  .           ?"
    set ans [tk_messageBox -icon question -type yesno -title "" -message "$msg"]
    if {$ans eq "yes"} {
        sendmail $body
    }
}
proc bytesFmt {x} {
    set k 0
    set y $x
    if {$y < 1000} {
        return [format "%7.0f " $y]
    }
    for {set k 0} { ($k <= 7) && (double($y) > 999.0) } {incr k} {
        set y [expr double($y) / 1024.0]
    }
    # "E" ("exa-", "-")  ,  2   60
    return [format "%7.3f%1s" $y [lindex { " " "k" "M" "G" "T" "P" "E" } $k]]
}
#   ,   
proc systemInfo {} {
    set info {}
    set osdescr {}
    set phys 0
    set avail 0
    set cpufreq 0
    set numofcpus 1
    set lang ""
 try_eval {
  if {[windowsPlatform]} {
    if {$::tcl_platform(os) ne "Windows NT" || [catch {package require twapi}]} {
        return "\[    \]\n:OS:$::tcl_platform(os) $::tcl_platform(osVersion)"
    }
    #catch {package require twapi}
    set osdescr [twapi::get_os_description]
    set meminfo [twapi::get_memory_info -totalphysical -availphysical]
    set phys    [string trim [bytesFmt [lindex $meminfo 1]]]
    set avail   [string trim [bytesFmt [lindex $meminfo 3]]]
    set cpufreq [lindex [twapi::get_processor_info {} -processorspeed] 1]
    set numofcpus 1
    catch {set numofcpus $::env(NUMBER_OF_PROCESSORS)}
    catch {set lang [twapi::map_langid_to_name [twapi::get_user_ui_langid]]}
  } else { # Linux/Unix
    # OS description
    catch "exec uname -a" osdescr

    # memory information
    set freeregexpr {^[a-zA-Z\s:]+(\d+)[\s\d]+[-/+a-zA-Z\s:]+\d+\s+(\d+).*$}
    #                          phys^^^                   used^^^free^^^
    catch "exec free -m" freeres
    regexp $freeregexpr $freeres skip phys avail
    set phys "$phys M"
    set avail "$avail M"

    # cpu information
    set cpumhzregexpr {^[a-zA-Z\s:]+(\d+\.?\d*).*$}
    #                             MHz^^^^^^^^^
    catch "exec grep MHz /proc/cpuinfo" cpumhz
    regexp $cpumhzregexpr $cpumhz skip cpufreq
    set n [regsub -all "\n" $cpumhz "." res]
    set numofcpus [incr n]
    catch {set lang $::env(LANG)}
  }
    if {![lempty $osdescr]} {set info ":OS:$osdescr"}
    if {$phys!=0}    {append info "\n:Total memory:$phys"}
    if {$avail!=0}   {append info "\n:Avail memory:$avail"}
    if {$cpufreq!=0} {append info "\n:CPU frequency:$cpufreq Mhz"}
    if {$numofcpus > 1} {
        append info "\n:Number of CPUs:$numofcpus"
    }
    if {$lang ne ""} {
        append info "\n:Language:$lang"
    }
    if {[info exists ::wine_used]} {
        append info "\n:Wine:[expr {($::wine_used) ? "yes":"no"}]"
    }
    append info [netInfo]
 } {
    return "$info\n   :\n$errorInfo\n"
 }
    return $info
}
proc netInfo {} {
    set res {}
  try_eval {
    if {[windowsPlatform]} {
        if {[catch {package require twapi}] || ![twapi::min_os_version 5]} {return}
        array set status_labels {
            nonoperational "Disconnected"
            disconnected   "Disconnected"
            wanconnecting  "Connecting"
            wanconnected   "Connected"
            operational    "Connected"
        }
        array set netinfo [twapi::get_network_info -all]
        append res "\n:DNS suffix:$netinfo(-domain)"
        append res "\n:Hostname:$netinfo(-hostname)"
        foreach ifx [twapi::get_netif_indices] {
            array set netif [twapi::get_netif_info $ifx -all]
            set netif(-dnsservers) [join $netif(-dnsservers) ", "]
            if {[info exists status_labels($netif(-operstatus))]} {
                set netif(-operstatus) $status_labels($netif(-operstatus))
            }
            set addrs {}
            foreach addr $netif(-ipaddresses) {
                lappend addrs "[lindex $addr 0] netmask [lindex $addr 1]"
            }
            set netif(-ipaddresses) [join $addrs ", "]
            set desc $netif(-description)
            append res "\n"
            append res "\n:Adapter:$desc"
            foreach {opt label} {
                -operstatus     ":Status:"
                -mtu            ":MTU size:"
                -type           ":Interface type:"
                -speed          ":Speed:"
                -ipaddresses    ":IP address:"
                -physicaladdress ":MAC address:"
                -defaultgateway ":Default gateway:"
                -dhcpserver     ":DHCP server:"
                -dnsservers     ":DNS servers:"
            } {
                catch {
                    if {$opt eq "-speed"} {
                        set speed [expr $netif($opt) / 1000000]
                        append speed " Mbit/s"
                        append res "\n$label$speed"
                    } else {
                        append res "\n$label$netif($opt)"
                    }
                }
            }
            if {[info exists ::bgtraf($desc)]} {
                foreach {tin tout} $::bgtraf($desc) {break}
                set tin [lindex [kbit2mbit $tin] 0]
                set tout [lindex [kbit2mbit $tout] 0]
                append res "\n:Traffic in:$tin Mbit/s"
                append res "\n:Traffic out:$tout Mbit/s"
            }
        }
    } else { # Linux/Unix
        # hostname and domain
        catch "exec hostname --domain" domain
        catch "exec hostname --short"  hostname
        if {[macPlatform]} {
            catch "exec hostname" hostname
            set domain $hostname
        }
        append res "\n:DNS suffix:$domain"
        append res "\n:Hostname:$hostname"
        if {![macPlatform]} {
        # list of DNS servers
        set resolvid [open "/etc/resolv.conf"]
        set resolv   [read $resolvid]
        close $resolvid
        set resexp {nameserver\s+([.0-9]+)}
        set dnssep ""
        set dnss   ""
        foreach str [split $resolv \n] {
            if {[regexp $resexp $str skip dnsip]} {
                append dnss "$dnssep $dnsip"
                set dnssep ","
            }
        }
        # list of network interfaces
        set devfid [open "/proc/net/dev"]
        set devs   [read $devfid]
        close $devfid
        set devreg {^\s+([a-zA-Z0-9]+):.*$}
        #         devname^^^^^^^^^^^^
        set ifconfigreg  {^.*Link encap:([a-zA-Z ]+).*?HWaddr.*?([0-9a-fA-F:]+).*?inet addr:([.0-9]+).*Mask:([.0-9]+).*MTU:([0-9]+).*$}
        set ifconfigreg1 {^.*Link encap:([a-zA-Z ]+).*MTU:([0-9]+).*$}
        foreach dev [split $devs \n] {
            if {[regexp $devreg $dev skip devname]} {
                set status  ""
                set mtu     ""
                set type    ""
                set ip      ""
                set mask    ""
                set gw      ""
                set dnssout ""
                set ipmask  ""
                set macaddr ""
                catch "exec /sbin/ifconfig $devname" ifcout
                if {[regexp $ifconfigreg $ifcout skip type macaddr ip mask mtu]} {
                    set status "Connected"
                    set ipmask "$ip mask $mask"
                    if {![info exists ::configip] && $ip ne "127.0.0.1"} {set ::configip $ip}
                    # get default dateway
                    set routereg {^.*\n0.0.0.0\s+([.0-9]+).*$}
                    catch "exec /sbin/route -n" routeout
                    regexp $routereg $routeout skip gw
                    # set list of dns servers
                    if {$devname != "lo"} {set dnssout $dnss}
                } elseif {[regexp $ifconfigreg1 $ifcout skip type mtu]} {
                    set status "Disconnected"
                }
                append res "\n\n:Adapter:"
                append res "\n:Status:$status"
                append res "\n:MTU size:$mtu"
                append res "\n:Interface name:$devname"
                append res "\n:Interface type:$type"
                append res "\n:Speed:"
                append res "\n:IP address:$ipmask"
                append res "\n:MAC address:$macaddr"
                append res "\n:Default gateway:$gw"
                append res "\n:DHCP server:"
                append res "\n:DNS servers:$dnssout"
             }
         }
         } else {
            catch {append res [exec ifconfig]}
         }
     }
  } {
    return "$res\n    :\n$errorInfo\n"
  }
    return [encoding convertfrom cp1251 $res]
}
proc myIP {} {
    set myip ""
    catch {
        set external [socket www.botik.ru 80]
        set myip     [lindex [fconfigure $external -sockname] 0]
        close $external
    }
    return $myip
}
#     
#      .
proc diagnose {} {
    global whoami ping versions_http external_ip
    if {![windowsPlatform]} {return}
    if {[catch {package require twapi}] || ![twapi::min_os_version 5]} {return}
    set info        ""
    set shortinfo   ""
    set my_ip       [myIP]
    set external_ip ""
    set local_ips   {}
    array set netinfo [twapi::get_network_info -all]
    set ips      $netinfo(-ipaddresses)
    set dns      $netinfo(-dnsservers)
    set domain   $netinfo(-domain)
    set hostname $netinfo(-hostname)
    set dns1     192.168.255.1
    set dns2     192.168.255.2
    set ping(diag_errors) ""
    set hostnameMsg ""
    set domainMsg   ""
    set dnsSuffix   ""
    set thisParam   ""
    set block $ping(whoami,connBlocked)
    if {![lempty $block] && $block ne "0"} {
        append info "  "
        if {$block ne "1"} {
            set block [string map {\n " "} $block]
            append info "   : $block"
        }
        append info ".\n\n"
        append shortinfo "*  .\n"
        lappend ping(diag_errors) 101
    }
    if {$ping(whoami,HostName) ne "" && $ping(whoami,IP) ne ""} {
        set realhostname [lindex [split $ping(whoami,HostName) "."] 0]; # real hostname
        if {[string compare -nocase $hostname $realhostname] != 0} {
            set namelength [string length $realhostname]
            set dnsSuffix [string range $ping(whoami,HostName) [expr $namelength+1] end]
            set dnsSuffix " ($dnsSuffix)"
            set hostnameMsg "   : $hostname,  : $realhostname "
            append shortinfo "*    .\n"
            lappend ping(diag_errors) 102
        }
    }
    if {$domain eq ""} {
        set domainMsg "   DNS-  $dnsSuffix. "
        append shortinfo "*    DNS-.\n"
        lappend ping(diag_errors) 104
    }
    set link62desc "   ,     \"\"\
                    :\
                http://www.botik.ru/2/abonents/faq/done/063-host-name-setting.html\n\n"
    if {$hostnameMsg ne "" || $domainMsg ne ""} {
        if {$hostnameMsg ne "" && $domainMsg ne ""} {
            set thisParam "  "
        } else {
            set thisParam "  "
        }
        append info $hostnameMsg $domainMsg $thisParam $link62desc
    }
    if {$dns eq ""} {
        append info "     DNS- ($dns1, $dns2).\n\n"
        append shortinfo "*   DNS-.\n"
        lappend ping(diag_errors) 105
    }
    #elseif {[lsearch -exact $dns $dns1] < 0 && [lsearch -exact $dns $dns2] < 0}
        #append info "  DNS-: $dns  : $dns1 / $dns2\n\n"
        #append shortinfo "*   DNS-.\n"
        #lappend ping(diag_errors) 106
    foreach ifx [twapi::get_netif_indices] {
        array set netif [twapi::get_netif_info $ifx -ipaddresses -operstatus -type]
        if {$netif(-type) eq "loopback"} {continue}
        if {$netif(-operstatus) ne "operational" && $netif(-operstatus) ne "wanconnected"} {continue}
        set addr  [lindex $netif(-ipaddresses) 0 0]
        #  IP-         (  ) IP-
        if {$addr ne "" && $addr ne "127.0.0.1" && $addr ne "0.0.0.0"} {
            if {[isLocal {} $addr]} {
                lappend local_ips $addr
            } else {
                append info "  IP-: $addr\n\n"
                append shortinfo "*  IP-: $addr.\n"
                lappend ping(diag_errors) 107
            }
        }
    }
    set nlocal [llength $local_ips]
    if {$ping(whoami,IP) ne ""} {
        set external_ip $ping(whoami,IP)
    } elseif {$my_ip ne "" && $my_ip ne "127.0.0.1"} {
        set external_ip $my_ip
    } elseif {$nlocal > 0 && [set lip0 [lindex $local_ips 0]] ne "127.0.0.1"} {
        set external_ip $lip0
    }
    if {$nlocal == 0} {
        append info "        ,      \"$::net(name)\".\n\n"
        append shortinfo "*   .\n"
        lappend ping(diag_errors) 109
    } elseif {$nlocal > 1 && [llength [lsort -unique $local_ips]] > 1} {
        #append info "     ($nlocal)    IP- [join $local_ips {, }]     .\n\n"
        append shortinfo "*   .\n"
        #lappend ping(diag_errors) 110
    }

    #   
    if {[array exists versions_http] && double($versions_http(BotikTester)) > double($ping(ver))} {
        append info "   : $ping(name) v.$ping(ver).\
                         : $ping(name) v.$versions_http(BotikTester).\
                          BotikTools   $ping(url_dload)\n\n"
        append shortinfo "*   .\n"
        lappend ping(diag_errors) 111
    }
    #    IPv6.
    #     ,      WinXP.
    if {[info exists ping(ipv6enabled)] && ![WinNT6]} {
        append info "  IPv6.   .\
                     ,      :\
                \"netsh interface ipv6 uninstall\"\n\n"
        append shortinfo "*  IPv6.\n"
        lappend ping(diag_errors) 112
    }

    if {![lempty $ping(whoami,pingBlocked)] && $ping(whoami,pingBlocked) eq "1"} {
        append info "     Ping-.\
                         \
                      . ,     \
                      ,      ::\
                         ping-.\
                    ,  \
                     ,     Ping-.\
                          ,   \
                          \"\":\
                    http://www.botik.ru/2/abonents/faq/done/057-firewall-settings.html\n\n"
        append shortinfo "*     Ping-.\n"
        lappend ping(diag_errors) 113
    }
    if {[info exists ::bgtraf(res)] && [llength $::bgtraf(res)]} {
        append info $::bgtraf(res)\n
        append shortinfo "*   IP-.\n"
        lappend ping(diag_errors) 114
    }
    if {$shortinfo ne ""} {
        set shortinfo "***   :\n$shortinfo\n"
    }
    return [list $info $shortinfo]
}
proc _whoAmI {} {
    global whoami_ botik
    array unset whoami_
    if {[info exists ::ping(url_whoami)]} {
        set whoamiurl $::ping(url_whoami)
    } elseif {[info exists ::botikkey(url_whoami)]} {
        set whoamiurl $::botikkey(url_whoami)
    } elseif {[info exists ::whoamiurl]} {
        set whoamiurl $::whoamiurl
    } else {
        return
    }
    set botik(cli_time) ""
    catch {http::geturl $whoamiurl -handler whoamiHandler -timeout 10000}
}
proc ftime_l {} {
    return [clockFormat "%Y-%m-%d %X"]
}
proc whoamiHandler {socket token} {
    global whoami_ botik
    gets $socket line
    set spl   [split $line ":"]
    set kword [lindex $spl 1]
    set value [string range $line [expr [string length $kword] + 2] end]
    set whoami_($kword) $value
    if {$kword eq "Time"} {
        set botik(cli_time) [ftime_l]
    }
    return
}
proc tokoi {str} {
    return [encoding convertto koi8-r $str]
}
#    (   )
proc getEnv {} {
    set globals [lsort [info globals]]
    set res     ""
    set vars    {}
    set arrays  {}
    set exclusions {bkOptions env errorInfo help wReport whoami_ bkLog \
                    conn zooms types states complex divs grid ids inlayer \
                    newID objects parameters srvids tags zhidden}
    foreach var $globals {
        if {[lcontain $exclusions $var]} {continue}
        if {[string match tcl_* $var]} {continue}
        if {[string match tk_* $var]} {continue}
        if {[string match auto_* $var]} {continue}
        if {[string match arg* $var]} {continue}
        if {[array exists ::$var]} {
            lappend arrays $var
        } else {
            lappend vars $var
        }
    }
    foreach var $vars {
        append res "-> [list set $var [set ::$var]]\n"
    }
    foreach var $arrays {
        append res "-> [list array set $var [array get ::$var]]\n"
    }
    return $res
}
